@@ -1,4 +1,4 @@
-use crate::types::UnsignedLike;
+use crate::data_types::UnsignedLike;
 
 const BIT_MASKS: [u8; 8] = [
   0x80,
@@ -56,7 +56,21 @@ pub fn bits_to_bytes(bits: Vec<bool>) -> Vec<u8> {
   res
 }
 
-pub fn bits_to_usize_truncated(bits: &[bool], max_depth: u32) -> usize {
+pub fn bytes_to_bits(bytes: Vec<u8>) -> Vec<bool> {
+  let mut res = Vec::with_capacity(bytes.len() * 8);
+  for b in bytes {
+    for &mask in &BIT_MASKS {
+      res.push(b & mask > 0);
+    }
+  }
+  res
+}
+
+pub fn bits_to_usize(bits: &[bool]) -> usize {
+  bits_to_usize_truncated(bits, bits.len())
+}
+
+pub fn bits_to_usize_truncated(bits: &[bool], max_depth: usize) -> usize {
   if max_depth < 1 {
     return 0;
   }
@@ -71,12 +85,12 @@ pub fn bits_to_usize_truncated(bits: &[bool], max_depth: u32) -> usize {
   res
 }
 
-pub fn usize_truncated_to_bits(x: usize, max_depth: u32) -> Vec<bool> {
+pub fn usize_truncated_to_bits(x: usize, max_depth: usize) -> Vec<bool> {
   if max_depth < 1 {
     return Vec::new();
   }
 
-  let mut res = Vec::with_capacity(max_depth as usize);
+  let mut res = Vec::with_capacity(max_depth);
   for i in 0..max_depth {
     res.push((x >> (max_depth - i - 1)) & 1 > 0);
   }
